@@ -25,12 +25,13 @@
 #' Scientific Reports, 12(20005).
 #' ISSN  20452322, \doi{10.1038/s41598-022-20991-1}.
 #'
-#' @return a vector containing the ratio statistics, an object of class
-#' \code{mus}. The length of the vector is equal to the number of observations
-#' considered, unless ties are present in the dataset. In that case, the
-#' duplicates are removed. Optionally, if \code{Nq} is \code{TRUE}, the function
-#' returns a list containing both the ratio statistics and the adjacency
-#' matrix \code{N^q}.
+#' @return the principal output of this function is a vector containing the
+#' ratio statistics, an object of class \code{mus}. The length of the vector is
+#' equal to the number of observations considered, unless ties are present in
+#' the dataset. In that case, the duplicates are removed. Optionally, if
+#' \code{Nq} is \code{TRUE}, the function returns an object of class
+#' \code{mus_Nq}, a list containing both the ratio statistics \code{mus} and the
+#' adjacency matrix \code{NQ}.
 #
 #' @export
 #'
@@ -57,6 +58,8 @@ compute_mus <- function(X = NULL,
 
   D <- NULL
 
+  #####################################################
+  # data, no dist_mat
   if (is.null(dist_mat)) {
     if(any(is.na(X))){
       stop("There are missing values in the provided dataset.
@@ -70,6 +73,7 @@ Please remove the problematic observations and try again.",
     check <- 0
     check <- duplicated(X)
 
+    # remove duplicates
     if (sum(check) > 0) {
       X <- X[-which(check),]
       n <- nrow(X)
@@ -86,29 +90,47 @@ Please remove the problematic observations and try again.",
       )
     }
 
+
+
+
     if (!Nq) {
       K   <- FNN::get.knn(X, k = n2)
       mus <- K$nn.dist[, n2] / K$nn.dist[, n1]
 
+      attr(mus, "upper_D") <- D
+      attr(mus, "n1") <- n1
+      attr(mus, "n2") <- n2
+      structure(mus, class = c("mus", class(mus)))
+
     } else{
       K   <- FNN::get.knn(X, k = max(n2, q))
       mus <- K$nn.dist[, n2] / K$nn.dist[, n1]
       NQ  <- matrix(0, n, n)
-
       for (h in 1:n) {
         NQ[h, K$nn.index[h,]] <- 1
       }
-      mus <- list(mus = mus, NQ = NQ)
+      attr(mus, "upper_D") <- D
+      attr(mus, "n1") <- n1
+      attr(mus, "n2") <- n2
+      mus <- structure(mus, class = c("mus", class(mus)))
 
+      mus_nq <- list(mus = mus, NQ = NQ)
+      attr(mus_nq, "upper_D") <- D
+      attr(mus_nq, "n1") <- n1
+      attr(mus_nq, "n2") <- n2
+      attr(mus_nq, "q")  <- q
+      structure(mus_nq, class = c("mus_Nq", class(mus_nq)))
     }
 
-    attr(mus, "upper_D") <- D
 
-  } else {
+    #################################
+    } else {
+    # dist_mat is passed
+    #################################
     # checks on dist_mat
 
     # if it is of class dist, then transform it into a matrix
-    if(inherits(dist_mat,"dist")){
+    if(inherits(dist_mat,"dist") | inherits(dist_mat,"dissimilarity")){
       dist_mat <- as.matrix(dist_mat)
     }
 
@@ -124,52 +146,58 @@ Please provide a valid distance matrix",
            call. = FALSE)
     }
     # NA?
-    if(any(is.na(X))){
+    if(any(is.na(dist_mat))){
       stop("There are missing values in the provided distance matrix.
 Please remove the problematic observations and try again.",
            call. = FALSE)
     }
-    n0    <- nrow(dist_mat)
+
+    n <- n0 <- nrow(dist_mat)
     dummy <- dist_mat
     dummy[lower.tri(dummy, diag = TRUE)] <- -1
-    inds     <- unique(which(dummy == 0, arr.ind = TRUE)[, 2])
+    inds  <- unique(which(dummy == 0, arr.ind = TRUE)[, 2])
 
     if (length(inds) > 0) {
       dist_mat <- dist_mat[-inds, -inds]
       n        <- nrow(dist_mat)
 
       warning(
-        paste0(
-          "\n  Duplicates are present and will be removed.
-             Original sample size: ",
-          n0,
-          ". New sample size: ",
-          n,
-          "."
+        paste0("\nDuplicates are present and will be removed.\n",
+               "Original sample size: ", n0,". New sample size: ",
+          n,"."
         )
       )
     }
 
-    if (!Nq) {
       sDistMat <- apply(dist_mat, 1, function(x)
         sort(x, index = TRUE))
       mus      <- unlist(lapply(sDistMat,
                                 function(z)
                                   z$x[n2 + 1] / z$x[n1 + 1]))
+    if (!Nq) {
+
+      attr(mus, "n1") <- n1
+      attr(mus, "n2") <- n2
+      structure(mus, class = c("mus", class(mus)))
+
+    }else{
 
-    } else{
       NQ       <- matrix(0, n, n)
       for (h in 1:n) {
-        NQ[h, (sDistMat[[h]]$ix)[2:q]] <- 1
+        NQ[h, (sDistMat[[h]]$ix)[2:(q+1)]] <- 1
       }
-      mus <- list(mus = mus, Nq = NQ)
 
+      attr(mus, "n1") <- n1
+      attr(mus, "n2") <- n2
+      mus <- structure(mus, class = c("mus", class(mus)))
+      mus_nq <- list(mus = mus, NQ = NQ)
+      attr(mus_nq, "n1") <- n1
+      attr(mus_nq, "n2") <- n2
+      attr(mus_nq, "q")  <- q
+      structure(mus_nq, class = c("mus_Nq", class(mus_nq)))
+
+      }
     }
-  }
-
-  attr(mus, "n1") <- n1
-  attr(mus, "n2") <- n2
-  structure(mus, class = c("mus", class(mus)))
 
 }
 
@@ -201,6 +229,33 @@ print.mus <- function(x, ...) {
   }
 
 
+#' @name compute_mus
+#'
+#' @param x object of class \code{mus_Nq}, obtained from the
+#' function \code{compute_mus()}.
+#' @param ... ignored.
+#'
+#' @export
+print.mus_Nq <- function(x, ...) {
+
+  if(is.list(x)){ #check if the mus object contains Nq
+    nn <- length(x[[1]])
+  }else{
+    nn <- length(x)
+  }
+
+  cat("List containing the ratio statistics mu's and the Nq adjacency matrix\n")
+
+  cat(paste0("NN orders: n1 = ", attr(x, "n1"), ", n2 = ",
+             attr(x, "n2"), "\n"))
+  cat(paste0("Sample size: ", nn, "\n"))
+  if (!is.null(attr(x, "upper_D"))) {
+    cat(paste0("Nominal Dimension: ", attr(x, "upper_D"), "\n"))
+  }
+  cat(paste0("NNs considered: q = ", attr(x, "q"), "\n"))
+
+  invisible(x)
+}
 
 
 #' @name compute_mus
